from django import forms
from django.contrib import admin
from django.forms.extras.widgets import SelectDateWidget

from ckeditor_link.admin import DjangoLinkAdmin

from .models import TestModel, LinkModel


class LinkModelForm(forms.ModelForm):
    when = forms.DateField(widget=SelectDateWidget, required=False)


class CKLinkModelAdmin(DjangoLinkAdmin):
    form = LinkModelForm

admin.site.register(TestModel)
admin.site.register(LinkModel, CKLinkModelAdmin)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.test import Client
from django.test import TestCase

from ckeditor_link.tests.utils.django_utils import create_superuser
from ckeditor_link.tests.utils.selenium_utils import SeleniumTestCase, CustomWebDriver
from ckeditor_link.tests.test_app.models import TestModel, LinkModel


class ckeditor_linkDialogTests(SeleniumTestCase):
    fixtures = ['test_app.json', ]

    def setUp(self):
        self.test_object = TestModel.objects.get(pk=2)

    def tearDown(self):
        pass

    def test_tag_link_target_class_value(self):
        """
        does it transform everything as it should?
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object.id])
        response = client.get(url)
        content = response.content
        # check it!

    def test_tag_no_destruction_of_existing_links(self):
        """
        normal existing <a href="xx" should not be tinkered with
        """
        pass

    def test_tag_robustness(self):
        """
        can it handle LinkModel with for example no .get_class method?
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object.id])
        response = client.get(url)
        content = response.content
        # check it!



# -*- coding: utf-8 -*-
from time import sleep

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.expected_conditions import visibility_of
from selenium.webdriver.support.wait import WebDriverWait

from ckeditor_link.tests.utils.selenium_utils import SeleniumTestCase, CustomWebDriver
from ckeditor_link.tests.test_app.models import TestModel, LinkModel


class ckeditor_linkEditorTests(SeleniumTestCase):
    fixtures = ['test_app.json', ]
    username = 'admin'
    password = 'admin'

    def setUp(self):
        superuser = User.objects.create_superuser(self.username, 'admin@free.fr', self.password)
        self.existing = TestModel.objects.get(pk=1)
        # Instantiating the WebDriver will load your browser
        self.wd = CustomWebDriver()

    def tearDown(self):
        self.wd.quit()

    def test_app_index_get(self):
        # if this fails, everything is probably broken.
        self.login()
        self.open(reverse('admin:index'))
        self.wd.find_css(".app-test_app")

    def test_editor_has_button_dialog_opens_has_form(self):
        self.login()
        sleep(1)  # must be a better solution!
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))
        sleep(1)  # argh
        button = self.wd.wait_for_css(".cke_button__djangolink")
        button[0].click()
        dialog_title = self.wd.wait_for_css(".cke_dialog_title")
        sleep(1)  # argh
        iframe = self.wd.find_css(".cke_dialog_ui_html")
        self.wd.switch_to.frame(iframe)
        target = self.wd.wait_for_css("#id_target")

    def test_dialog_form_validation(self):
        self.login()
        sleep(1)  # must be a better solution!
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))
        sleep(1)  # argh
        button = self.wd.wait_for_css(".cke_button__djangolink")
        button[0].click()
        self.wd.wait_for_css(".cke_dialog_title")
        sleep(1)  # argh
        iframe = self.wd.find_css(".cke_dialog_ui_html")
        self.wd.switch_to.frame(iframe)
        email = self.wd.wait_for_css("#id_email")
        email.send_keys('what-foo')
        self.wd.switch_to.default_content()
        ok = self.wd.wait_for_css(".cke_dialog_ui_button_ok")
        ok.click()
        self.wd.switch_to.frame(iframe)
        self.wd.wait_for_css(".field-email .errorlist")
        email.send_keys('root@example.com')
        self.wd.switch_to.default_content()
        ok = self.wd.wait_for_css(".cke_dialog_ui_button_ok")
        ok.click()
        sleep(1)  # argh
        try:
            title = self.wd.find_css(".cke_dialog_title")
            self.assertFalse(title.is_displayed())
        except NoSuchElementException:
            # ok if removed from the DOM!!
            pass

    def test_dialog_submit_and_link_attrs(self):
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))

    def test_remove_foreign_key_attrs(self):
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))


import importlib

from django.conf import settings
from django import template
from django.template.defaultfilters import stringfilter


try:
    module_name, class_name = settings.CKEDITOR_LINK_MODEL.rsplit(".", 1)
    my_module = importlib.import_module(module_name)
    ckeditor_link_class = getattr(my_module, class_name)
except ImportError:
    ckeditor_link_class = None


register = template.Library()


@register.filter
@stringfilter
def ckeditor_link_add_links(html):
    # lxml is not a dependency, but needed for this tag.
    from lxml.html import fragment_fromstring, tostring
    if not ckeditor_link_class:
        # TODO: use some log thing
        if settings.DEBUG:
            print "Warning: CKEDITOR_LINK_MODEL (%s) could not be imported!?" % settings.CKEDITOR_LINK_MODEL
        return html
    fragment = fragment_fromstring("<div>" + html + "</div>")
    links = fragment.cssselect('a')
    for link in links:
        if link.get('data-ckeditor-link', None):
            link.attrib.pop('data-ckeditor-link')
            kwargs = {}
            dummy_link = ckeditor_link_class()
            for key, value in link.items():
                if key.startswith('data-'):
                    new_key = key.replace('data-', '', 1)
                    # will be removed!
                    if new_key == 'page_2':
                        new_key = 'page'
                    # until here
                    if hasattr(dummy_link, new_key):
                        if hasattr(dummy_link, new_key + "_id"):
                            # set fk directly
                            new_key = new_key + "_id"
                            if not value:
                                value = None
                        kwargs[new_key] = value
                        link.attrib.pop(key)
            real_link = ckeditor_link_class(**kwargs)
            link.set('href', real_link.get_link())
            if (getattr(real_link, 'get_target')):
                link.set('target', real_link.get_target())
            if (getattr(real_link, 'get_css_class')):
                link.set('class', real_link.get_css_class())
    return tostring(fragment)



from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.db import models
from ckeditor.fields import RichTextField
# from djangocms_text_ckeditor.fields import HTMLField


@python_2_unicode_compatible
class TestModel(models.Model):
    title = models.CharField(max_length=255, )
    richtext = RichTextField(default='')
    richtext_second = RichTextField(default='')

    def __str__(self):
        return "%s" % self.title

    def get_absolute_url(self):
        return reverse('testmodel_detail', args=(self.id, ))


# @python_2_unicode_compatible
# class CMSTestModel(models.Model):
#     title = models.CharField(max_length=255, )
#     richtext = HTMLField()
#
#     def __str__(self):
#         return "%s" % self.title


@python_2_unicode_compatible
class LinkModelBase(models.Model):
    target = models.CharField(max_length=255, blank=True, default='', )
    external_url = models.CharField(max_length=255, blank=True, default='',)
    email = models.EmailField(blank=True, default='',)
    # http://stackoverflow.com/questions/12644142/prefill-a-datetimefield-from-url-in-django-admin
    when = models.DateField(blank=True, null=True)
    testmodel = models.ForeignKey(TestModel, null=True, default=None, blank=True)

    class Meta:
        abstract = True

    def __str__(self):
        return "LINK object: %s" % self.get_link()

    def get_link(self):
        if self.external_url:
            return self.external_url
        else:
            return "http://no-link-given.com/"

    def get_target(self):
        return "_blank"

    def get_css_class(self):
        return "no-css-class"


class LinkModel(LinkModelBase):
    pass
